@@ -1,9 +1,9 @@
-# This Source Code Form is subject to the terms of the Mozilla Public
-# License, v. 2.0. If a copy of the MPL was not distributed with this
-# file, You can obtain one at http://mozilla.org/MPL/2.0/.
-#
-# Copyright (C) 2017, James R. Barlow (https://github.com/jbarlow83/)
+# SPDX-FileCopyrightText: 2022 James R. Barlow
+# SPDX-License-Identifier: MPL-2.0
 
+"""Extract images embedded in PDF."""
+
+from __future__ import annotations
 
 from abc import ABC, abstractmethod
 from decimal import Decimal
@@ -11,20 +11,11 @@ from io import BytesIO
 from itertools import zip_longest
 from pathlib import Path
 from shutil import copyfileobj
-from typing import (
-    Any,
-    BinaryIO,
-    Callable,
-    NamedTuple,
-    Optional,
-    Tuple,
-    Type,
-    TypeVar,
-    Union,
-    cast,
-)
+from typing import Any, BinaryIO, Callable, NamedTuple, Sequence, TypeVar, cast
 
-from PIL import Image, ImageCms
+from deprecation import deprecated
+from PIL import Image
+from PIL.ImageCms import ImageCmsProfile
 
 from pikepdf import (
     Array,
@@ -40,31 +31,34 @@ from pikepdf import (
 )
 from pikepdf._exceptions import DependencyError
 from pikepdf._qpdf import Buffer
+from pikepdf._version import __version__
 from pikepdf.models import _transcoding
 
+T = TypeVar('T')
+
 
 class UnsupportedImageTypeError(Exception):
-    "This image is formatted in a way pikepdf does not supported."
+    """This image is formatted in a way pikepdf does not supported."""
 
 
 class NotExtractableError(Exception):
-    "Indicates that an image cannot be directly extracted."
+    """Indicates that an image cannot be directly extracted."""
 
 
 class HifiPrintImageNotTranscodableError(NotExtractableError):
-    "Image contains high fidelity printing information and cannot be extracted."
+    """Image contains high fidelity printing information and cannot be extracted."""
 
 
 class InvalidPdfImageError(Exception):
-    "This image is not valid according to the PDF 1.7 specification."
+    """This image is not valid according to the PDF 1.7 specification."""
 
 
-def array_str(value: Union[Object, str, list]):
+def _array_str(value: Object | str | list):
     """Simplify pikepdf objects to array of str. Keep Streams and dictionaries intact."""
 
-    def _array_str(item):
+    def _convert(item):
         if isinstance(item, (list, Array)):
-            return [_array_str(subitem) for subitem in item]
+            return [_convert(subitem) for subitem in item]
         if isinstance(item, (Stream, Dictionary, bytes, int)):
             return item
         if isinstance(item, (Name, str)):
@@ -73,28 +67,39 @@ def array_str(value: Union[Object, str, list]):
             return bytes(item)
         raise NotImplementedError(value)
 
-    result = _array_str(value)
+    result = _convert(value)
     if not isinstance(result, list):
         result = [result]
     return result
 
 
-def dict_or_array_dict(value):
+@deprecated(deprecated_in='5.5.0', removed_in='6.0.0', current_version=__version__)
+def array_str(value: Object | str | list):  # noqa: D103; pragma: no cover
+    return _array_str(value)
+
+
+def _ensure_list(value: list[Object] | Dictionary | Array) -> list[Object]:
+    """Ensure value is a list of pikepdf.Object, if it was not already.
+
+    To support DecodeParms which can be present as either an array of dicts or a single
+    dict. It's easier to convert to an array of one dict.
+    """
     if isinstance(value, list):
         return value
-    if isinstance(value, Dictionary):
-        return [value.as_dict()]
-    if isinstance(value, Array):
-        return list(value.as_list())
-    raise NotImplementedError(value)
+    return list(value.wrap_in_array().as_list())
 
 
-T = TypeVar('T')
+@deprecated(deprecated_in='5.5.0', removed_in='6.0.0', current_version=__version__)
+def dict_or_array_dict(
+    value: list[Object] | Dictionary | Array,
+) -> list[Object]:  # noqa: D103; pragma: no cover
+    return _ensure_list(value)
 
 
-def metadata_from_obj(
-    obj: Union[Dictionary, Stream], name: str, type_: Callable[[Any], T], default: T
-) -> Optional[T]:
+def _metadata_from_obj(
+    obj: Dictionary | Stream, name: str, type_: Callable[[Any], T], default: T
+) -> T | None:
+    """Retrieve metadata from a dictionary or stream, and ensure it is the expected type."""
     val = getattr(obj, name, default)
     try:
         return type_(val)
@@ -104,6 +109,13 @@ def metadata_from_obj(
     raise NotImplementedError('Metadata access for ' + name)
 
 
+@deprecated(deprecated_in='5.5.0', removed_in='6.0.0', current_version=__version__)
+def metadata_from_obj(
+    obj: Dictionary | Stream, name: str, type_: Callable[[Any], T], default: T
+) -> T | None:  # noqa: D103; pragma: no cover
+    return _metadata_from_obj(obj, name, type_, default)
+
+
 class PaletteData(NamedTuple):
     """Returns the color space and binary representation of the palette.
 
@@ -118,13 +130,14 @@ class PaletteData(NamedTuple):
 
 
 class PdfImageBase(ABC):
+    """Abstract base class for images."""
 
     SIMPLE_COLORSPACES = {'/DeviceRGB', '/DeviceGray', '/CalRGB', '/CalGray'}
     MAIN_COLORSPACES = SIMPLE_COLORSPACES | {'/DeviceCMYK', '/CalCMYK', '/ICCBased'}
     PRINT_COLORSPACES = {'/Separation', '/DeviceN'}
 
     @abstractmethod
-    def _metadata(self, name: str, type_: Type[T], default: T) -> T:
+    def _metadata(self, name: str, type_: Callable[[Any], T], default: T) -> T:
         """Get metadata for this image type."""
 
     @property
@@ -139,31 +152,31 @@ class PdfImageBase(ABC):
 
     @property
     def image_mask(self) -> bool:
-        """``True`` if this is an image mask."""
+        """Return ``True`` if this is an image mask."""
         return self._metadata('ImageMask', bool, False)
 
     @property
-    def _bpc(self) -> Optional[int]:
+    def _bpc(self) -> int | None:
         """Bits per component for this image (low-level)."""
         return self._metadata('BitsPerComponent', int, 0)
 
     @property
     def _colorspaces(self):
         """Colorspace (low-level)."""
-        return self._metadata('ColorSpace', array_str, [])
+        return self._metadata('ColorSpace', _array_str, [])
 
     @property
     def filters(self):
         """List of names of the filters that we applied to encode this image."""
-        return self._metadata('Filter', array_str, [])
+        return self._metadata('Filter', _array_str, [])
 
     @property
     def decode_parms(self):
         """List of the /DecodeParms, arguments to filters."""
-        return self._metadata('DecodeParms', dict_or_array_dict, [])
+        return self._metadata('DecodeParms', _ensure_list, [])
 
     @property
-    def colorspace(self) -> Optional[str]:
+    def colorspace(self) -> str | None:
         """PDF name of the colorspace that best describes this image."""
         if self.image_mask:
             return None  # Undefined for image masks
@@ -193,19 +206,25 @@ class PdfImageBase(ABC):
             return 1 if self.image_mask else 8
         return self._bpc
 
-    @property
+    @property  # type: ignore
     @abstractmethod
+    @deprecated(
+        deprecated_in='5.5.0',
+        removed_in='6.0.0',
+        current_version=__version__,
+        details="Use isinstance(im, PdfInlineImage) instead",
+    )
     def is_inline(self) -> bool:
-        """Is this an inline image?"""
+        """Test if this an inline image."""
 
     @property
     @abstractmethod
-    def icc(self) -> Optional[ImageCms.ImageCmsProfile]:
-        """Returns ICC profile for this image if one is defined."""
+    def icc(self) -> ImageCmsProfile | None:
+        """Return ICC profile for this image if one is defined."""
 
     @property
     def indexed(self) -> bool:
-        """``True`` if the image has a defined color palette."""
+        """Check if the image has a defined color palette."""
         return '/Indexed' in self._colorspaces
 
     def _colorspace_has_name(self, name):
@@ -221,16 +240,16 @@ class PdfImageBase(ABC):
 
     @property
     def is_device_n(self) -> bool:
-        """``True`` if image has a /DeviceN (complex printing) colorspace."""
+        """Check if image has a /DeviceN (complex printing) colorspace."""
         return self._colorspace_has_name('/DeviceN')
 
     @property
     def is_separation(self) -> bool:
-        """``True`` if image has a /DeviceN (complex printing) colorspace."""
+        """Check if image has a /DeviceN (complex printing) colorspace."""
         return self._colorspace_has_name('/Separation')
 
     @property
-    def size(self) -> Tuple[int, int]:
+    def size(self) -> tuple[int, int]:
         """Size of image as (width, height)."""
         return self.width, self.height
 
@@ -251,12 +270,11 @@ class PdfImageBase(ABC):
 
     @property
     def mode(self) -> str:
-        """``PIL.Image.mode`` equivalent for this image, where possible
+        """``PIL.Image.mode`` equivalent for this image, where possible.
 
         If an ICC profile is attached to the image, we still attempt to resolve a Pillow
         mode.
         """
-
         m = ''
         if self.is_device_n:
             m = 'DeviceN'
@@ -287,7 +305,9 @@ class PdfImageBase(ABC):
 
     @property
     def filter_decodeparms(self):
-        """PDF has a lot of optional data structures concerning /Filter and
+        """Return normalized the Filter and DecodeParms data.
+
+        PDF has a lot of possible data structures concerning /Filter and
         /DecodeParms. /Filter can be absent or a name or an array, /DecodeParms
         can be absent or a dictionary (if /Filter is a name) or an array (if
         /Filter is an array). When both are arrays the lengths match.
@@ -300,9 +320,8 @@ class PdfImageBase(ABC):
         return list(zip_longest(self.filters, self.decode_parms, fillvalue={}))
 
     @property
-    def palette(self) -> Optional[PaletteData]:
-        """Retrieves the color palette for this image if applicable."""
-
+    def palette(self) -> PaletteData | None:
+        """Retrieve the color palette for this image if applicable."""
         if not self.indexed:
             return None
         try:
@@ -335,14 +354,13 @@ class PdfImageBase(ABC):
         """Convert this PDF image to a Python PIL (Pillow) image."""
 
     @staticmethod
-    def _remove_simple_filters(obj, filters):
+    def _remove_simple_filters(obj: Stream, filters: Sequence[str]):
         """Remove simple lossless compression where it appears.
 
         Args:
-            obj (pikepdf.Stream): the compressed object
-            filters (list of str): all files on the data
+            obj: the compressed object
+            filters: all files on the data
         """
-
         COMPLEX_FILTERS = {
             '/DCTDecode',
             '/JPXDecode',
@@ -377,7 +395,7 @@ class PdfImageBase(ABC):
 
 
 class PdfImage(PdfImageBase):
-    """Support class to provide a consistent API for manipulating PDF images
+    """Support class to provide a consistent API for manipulating PDF images.
 
     The data structure for images inside PDFs is irregular and complex,
     making it difficult to use without introducing errors for less
@@ -386,7 +404,11 @@ class PdfImage(PdfImageBase):
     Pillow imaging library.
     """
 
+    obj: Stream
+    _icc: ImageCmsProfile | None
+
     def __new__(cls, obj):
+        """Construct a PdfImage... or a PdfJpxImage if that is what we really are."""
         instance = super().__new__(cls)
         instance.__init__(obj)
         if '/JPXDecode' in instance.filters:
@@ -394,14 +416,13 @@ class PdfImage(PdfImageBase):
             instance.__init__(obj)
         return instance
 
-    def __init__(self, obj):
-        """Construct a PDF image from a Image XObject inside a PDF
+    def __init__(self, obj: Stream):
+        """Construct a PDF image from a Image XObject inside a PDF.
 
         ``pim = PdfImage(page.Resources.XObject['/ImageNN'])``
 
         Args:
-            obj (pikepdf.Object): an Image XObject
-
+            obj: an Image XObject
         """
         if isinstance(obj, Stream) and obj.stream_dict.get("/Subtype") != "/Image":
             raise TypeError("can't construct PdfImage from non-image")
@@ -415,7 +436,7 @@ class PdfImage(PdfImageBase):
 
     @classmethod
     def _from_pil_image(cls, *, pdf, page, name, image):  # pragma: no cover
-        """Insert a PIL image into a PDF (rudimentary)
+        """Insert a PIL image into a PDF (rudimentary).
 
         Args:
             pdf (pikepdf.Pdf): the PDF to attach the image to
@@ -423,7 +444,6 @@ class PdfImage(PdfImageBase):
             name (str or pikepdf.Name): the name to set the image
             image (PIL.Image.Image): the image to insert
         """
-
         data = image.tobytes()
 
         imstream = Stream(pdf, data)
@@ -442,11 +462,17 @@ class PdfImage(PdfImageBase):
         return cls(imstream)
 
     def _metadata(self, name, type_, default):
-        return metadata_from_obj(self.obj, name, type_, default)
+        return _metadata_from_obj(self.obj, name, type_, default)
 
-    @property
+    @property  # type: ignore
+    @deprecated(
+        deprecated_in='5.5.0',
+        removed_in='6.0.0',
+        current_version=__version__,
+        details="Use isinstance(im, PdfInlineImage) instead",
+    )
     def is_inline(self):
-        """``False`` for image XObject."""
+        """Return False since PdfImage cannot be inline."""
         return False
 
     @property
@@ -459,11 +485,10 @@ class PdfImage(PdfImageBase):
         raise NotImplementedError("Don't know how to find ICC stream for image")
 
     @property
-    def icc(self) -> Optional[ImageCms.ImageCmsProfile]:
+    def icc(self) -> ImageCmsProfile | None:
         """If an ICC profile is attached, return a Pillow object that describe it.
 
         Most of the information may be found in ``icc.profile``.
-
         """
         if self.colorspace not in ('/ICCBased', '/Indexed'):
             return None
@@ -472,7 +497,7 @@ class PdfImage(PdfImageBase):
             iccbuffer = iccstream.get_stream_buffer()
             iccbytesio = BytesIO(iccbuffer)
             try:
-                self._icc = ImageCms.ImageCmsProfile(iccbytesio)
+                self._icc = ImageCmsProfile(iccbytesio)
             except OSError as e:
                 if str(e) == 'cannot open profile from string':
                     # ICC profile is corrupt
@@ -482,7 +507,7 @@ class PdfImage(PdfImageBase):
         return self._icc
 
     def _extract_direct(self, *, stream: BinaryIO) -> str:
-        """Attempt to extract the image directly to a usable image file
+        """Attempt to extract the image directly to a usable image file.
 
         If there is no way to extract the image without decompressing or
         transcoding then raise an exception. The type and format of image
@@ -529,7 +554,6 @@ class PdfImage(PdfImageBase):
 
     def _extract_transcoded_1248bits(self) -> Image.Image:
         """Extract an image when there are 1/2/4/8 bits packed in byte data."""
-
         stride = 0  # tell Pillow to calculate stride from line width
         scale = 0 if self.mode == 'L' else 1
         if self.bits_per_component in (2, 4):
@@ -606,7 +630,7 @@ class PdfImage(PdfImageBase):
         return im
 
     def _extract_to_stream(self, *, stream: BinaryIO) -> str:
-        """Attempt to extract the image to a stream
+        """Extract the image to a stream.
 
         If possible, the compressed data is extracted and inserted into
         a compressed image file format without transcoding the compressed
@@ -619,7 +643,6 @@ class PdfImage(PdfImageBase):
         Returns:
             The file format extension.
         """
-
         try:
             return self._extract_direct(stream=stream)
         except NotExtractableError:
@@ -645,9 +668,9 @@ class PdfImage(PdfImageBase):
         raise UnsupportedImageTypeError(repr(self))
 
     def extract_to(
-        self, *, stream: Optional[BinaryIO] = None, fileprefix: str = ''
+        self, *, stream: BinaryIO | None = None, fileprefix: str = ''
     ) -> str:
-        """Attempt to extract the image directly to a usable image file
+        """Extract the image directly to a usable image file.
 
         If possible, the compressed data is extracted and inserted into
         a compressed image file format without transcoding the compressed
@@ -662,7 +685,6 @@ class PdfImage(PdfImageBase):
         Images might be saved as any of .png, .jpg, or .tiff.
 
         Examples:
-
             >>> im.extract_to(stream=bytes_io)
             '.png'
 
@@ -679,7 +701,6 @@ class PdfImage(PdfImageBase):
             appropriate extension. If no *fileprefix*, then an extension
             indicating the file type.
         """
-
         if bool(stream) == bool(fileprefix):
             raise ValueError("Cannot set both stream and fileprefix")
         if stream:
@@ -724,7 +745,7 @@ class PdfImage(PdfImageBase):
 
         return im
 
-    def _generate_ccitt_header(self, data: bytes, icc: Optional[bytes] = None) -> bytes:
+    def _generate_ccitt_header(self, data: bytes, icc: bytes | None = None) -> bytes:
         """Construct a CCITT G3 or G4 header from the PDF metadata."""
         # https://stackoverflow.com/questions/2641770/
         # https://www.itu.int/itudoc/itu-t/com16/tiff-fx/docs/tiff6.pdf
@@ -779,9 +800,14 @@ class PdfImage(PdfImageBase):
 
 
 class PdfJpxImage(PdfImage):
-    """Support class for JPEG 2000 images. Implements the same API as :class:`PdfImage`."""
+    """Support class for JPEG 2000 images. Implements the same API as :class:`PdfImage`.
+
+    If you call PdfImage(object_that_is_actually_jpeg2000_image), pikepdf will return
+    this class instead, due to the check in PdfImage.__new__.
+    """
 
     def __init__(self, obj):
+        """Initialize a JPEG 2000 image."""
         super().__init__(obj)
         self._jpxpil = self.as_pil_image()
 
@@ -803,6 +829,11 @@ class PdfJpxImage(PdfImage):
 
     @property
     def _colorspaces(self):
+        """Return the effective colorspace of a JPEG 2000 image.
+
+        If the ColorSpace dictionary is present, the colorspace embedded in the
+        JPEG 2000 data will be ignored, as required by the specification.
+        """
         # (PDF 1.7 Table 89) If ColorSpace is present, any colour space
         # specifications in the JPEG2000 data shall be ignored.
         super_colorspaces = super()._colorspaces
@@ -816,6 +847,7 @@ class PdfJpxImage(PdfImage):
 
     @property
     def _bpc(self) -> int:
+        """Return 8, since bpc is not meaningful for JPEG 2000 encoding."""
         # (PDF 1.7 Table 89) If the image stream uses the JPXDecode filter, this
         # entry is optional and shall be ignored if present. The bit depth is
         # determined by the conforming reader in the process of decoding the
@@ -824,6 +856,7 @@ class PdfJpxImage(PdfImage):
 
     @property
     def indexed(self) -> bool:
+        """Return False, since JPEG 2000 should not be indexed."""
         # Nothing in the spec precludes an Indexed JPXDecode image, except for
         # the fact that doing so is madness. Let's assume it no one is that
         # insane.
@@ -861,13 +894,16 @@ class PdfInlineImage(PdfImageBase):
     }
     REVERSE_ABBREVS = {v: k for k, v in ABBREVS.items()}
 
+    _data: Object
+    _image_object: tuple[Object, ...]
+
     def __init__(self, *, image_data: Object, image_object: tuple):
-        """
+        """Construct wrapper for inline image.
+
         Args:
             image_data: data stream for image, extracted from content stream
             image_object: the metadata for image, also from content stream
         """
-
         # Convert the sequence of pikepdf.Object from the content stream into
         # a dictionary object by unparsing it (to bytes), eliminating inline
         # image abbreviations, and constructing a bytes string equivalent to
@@ -912,9 +948,11 @@ class PdfInlineImage(PdfImageBase):
         raise NotImplementedError(repr(obj))
 
     def _metadata(self, name, type_, default):
-        return metadata_from_obj(self.obj, name, type_, default)
+        return _metadata_from_obj(self.obj, name, type_, default)
 
     def unparse(self) -> bytes:
+        """Create the content stream bytes that reproduce this inline image."""
+
         def metadata_tokens():
             for metadata_obj in self._image_object:
                 unparsed = self._unparse_obj(
@@ -932,12 +970,20 @@ class PdfInlineImage(PdfImageBase):
 
         return b''.join(inline_image_tokens())
 
-    @property
+    @property  # type: ignore
+    @deprecated(
+        deprecated_in='5.5.0',
+        removed_in='6.0.0',
+        current_version=__version__,
+        details="Use isinstance(im, PdfInlineImage) instead",
+    )
     def is_inline(self) -> bool:
+        """Return True, since this is an inline image."""
         return True
 
     @property
     def icc(self):  # pragma: no cover
+        """Raise an exception since ICC profiles are not supported on inline images."""
         raise InvalidPdfImageError(
             "Inline images with ICC profiles are not supported in the PDF specification"
         )
@@ -970,17 +1016,25 @@ class PdfInlineImage(PdfImageBase):
         return img
 
     def as_pil_image(self) -> Image.Image:
+        """Return inline image as a Pillow Image."""
         return self._convert_to_pdfimage().as_pil_image()
 
-    def extract_to(self, *, stream: Optional[BinaryIO] = None, fileprefix: str = ''):
+    def extract_to(self, *, stream: BinaryIO | None = None, fileprefix: str = ''):
+        """Extract the inline image directly to a usable image file.
+
+        See:
+            :meth:`PdfImage.extract_to`
+        """
         return self._convert_to_pdfimage().extract_to(
             stream=stream, fileprefix=fileprefix
         )
 
     def read_bytes(self):
-        raise NotImplementedError("qpdf returns compressed")
-        # return self._data._inline_image_bytes()
+        """Return decompressed image bytes."""
+        # QPDF does not have an API to return this directly, so convert it.
+        return self._convert_to_pdfimage().read_bytes()
 
     def get_stream_buffer(self):
-        raise NotImplementedError("qpdf returns compressed")
-        # return memoryview(self._data.inline_image_bytes())
+        """Return decompressed stream buffer."""
+        # QPDF does not have an API to return this directly, so convert it.
+        return self._convert_to_pdfimage().get_stream_buffer()
