@@ -1,10 +1,5 @@
-/*
- * This Source Code Form is subject to the terms of the Mozilla Public
- * License, v. 2.0. If a copy of the MPL was not distributed with this
- * file, You can obtain one at http://mozilla.org/MPL/2.0/.
- *
- * Copyright (C) 2019, James R. Barlow (https://github.com/jbarlow83/)
- */
+// SPDX-FileCopyrightText: 2022 James R. Barlow
+// SPDX-License-Identifier: MPL-2.0
 
 #include <qpdf/Constants.h>
 #include <qpdf/Types.h>
@@ -23,8 +18,9 @@
 
 void init_embeddedfiles(py::module_ &m)
 {
-    py::class_<QPDFFileSpecObjectHelper, std::shared_ptr<QPDFFileSpecObjectHelper>>(
-        m, "AttachedFileSpec")
+    py::class_<QPDFFileSpecObjectHelper,
+        std::shared_ptr<QPDFFileSpecObjectHelper>,
+        QPDFObjectHelper>(m, "AttachedFileSpec")
         .def(py::init([](QPDF &q,
                           py::bytes data,
                           std::string description,
@@ -72,8 +68,6 @@ void init_embeddedfiles(py::module_ &m)
                 creation_date: PDF date string for when this file was creation.
                 mod_date: PDF date string for when this file was last modified.
             )~~~")
-        .def_property_readonly("obj",
-            [](QPDFFileSpecObjectHelper &spec) { return spec.getObjectHandle(); })
         .def_property("description",
             &QPDFFileSpecObjectHelper::getDescription,
             &QPDFFileSpecObjectHelper::setDescription,
@@ -138,11 +132,9 @@ void init_embeddedfiles(py::module_ &m)
             names.
             )~~~");
 
-    py::class_<QPDFEFStreamObjectHelper>(m, "AttachedFile")
-        .def_property_readonly("obj",
-            [](QPDFEFStreamObjectHelper &efstream) {
-                return efstream.getObjectHandle();
-            })
+    py::class_<QPDFEFStreamObjectHelper,
+        std::shared_ptr<QPDFEFStreamObjectHelper>,
+        QPDFObjectHelper>(m, "AttachedFile")
         .def_property_readonly("size",
             &QPDFEFStreamObjectHelper::getSize,
             "Get length of the attached file in bytes according to the PDF creator.")
