@@ -1,10 +1,5 @@
-/*
- * This Source Code Form is subject to the terms of the Mozilla Public
- * License, v. 2.0. If a copy of the MPL was not distributed with this
- * file, You can obtain one at http://mozilla.org/MPL/2.0/.
- *
- * Copyright (C) 2017, James R. Barlow (https://github.com/jbarlow83/)
- */
+// SPDX-FileCopyrightText: 2022 James R. Barlow
+// SPDX-License-Identifier: MPL-2.0
 
 #include <qpdf/Constants.h>
 #include <qpdf/Types.h>
@@ -26,12 +21,12 @@
 void Pl_PythonOutput::write(unsigned char *buf, size_t len)
 {
     py::gil_scoped_acquire gil;
-    ssize_t so_far = 0;
+    py::ssize_t so_far = 0;
     while (len > 0) {
         auto view_buffer  = py::memoryview::from_memory(buf, len);
         py::object result = this->stream.attr("write")(view_buffer);
         try {
-            so_far = result.cast<ssize_t>();
+            so_far = result.cast<py::ssize_t>();
         } catch (const py::cast_error &e) {
             throw py::type_error("Unexpected return type of write()");
         }
@@ -50,9 +45,5 @@ void Pl_PythonOutput::write(unsigned char *buf, size_t len)
 void Pl_PythonOutput::finish()
 {
     py::gil_scoped_acquire gil;
-    try {
-        this->stream.attr("flush")();
-    } catch (const py::attr_error &e) {
-        // Suppress
-    }
+    this->stream.attr("flush")();
 }
