@@ -1,3 +1,8 @@
+# SPDX-FileCopyrightText: 2022 James R. Barlow
+# SPDX-License-Identifier: CC0-1.0
+
+from __future__ import annotations
+
 import os
 import platform
 import sys
@@ -37,6 +42,13 @@ def outpdf(tmp_path):
     return tmp_path / 'out.pdf'
 
 
+@pytest.fixture
+def refcount():
+    if platform.python_implementation() == 'PyPy':
+        pytest.skip(reason="test isn't valid for PyPy")
+    return sys.getrefcount
+
+
 skip_if_pypy = pytest.mark.skipif(
     platform.python_implementation() == 'PyPy', reason="test isn't valid for PyPy"
 )
