@@ -13,6 +13,7 @@ from pathlib import Path
 from subprocess import run
 from typing import NamedTuple, Sequence
 
+import deprecation
 import PIL
 import pytest
 from conftest import needs_python_v
@@ -208,6 +209,7 @@ def test_lowlevel_replace_jpeg(congress, outdir):
     pdf.save(outdir / 'congress_gray.pdf')
 
 
+@deprecation.fail_if_not_removed
 def test_inline(inline):
     iimage, pdf = inline
     assert iimage.width == 8
@@ -238,6 +240,11 @@ def test_inline_extract(inline):
     assert im.size == (8, 8) and im.mode == iimage.mode
 
 
+def test_inline_read(inline):
+    iimage, _pdf = inline
+    assert iimage.read_bytes()[0:6] == b'\xff\xff\xff\x00\x00\x00'
+
+
 def test_inline_to_pil(inline):
     iimage, _pdf = inline
     im = iimage.as_pil_image()
@@ -501,6 +508,7 @@ def valid_random_palette_image_spec(
     return PaletteImageSpec(bpc, width, height, hival, colorspace, palette, imbytes)
 
 
+@deprecation.fail_if_not_removed
 @pytest.mark.parametrize(
     'filename,bpc,rgb',
     [
@@ -582,6 +590,7 @@ def test_bool_in_inline_image():
 @pytest.mark.skipif(
     not PIL_features.check_codec('jpg_2000'), reason='no JPEG2000 codec'
 )
+@deprecation.fail_if_not_removed
 def test_jp2(first_image_in):
     xobj, _pdf = first_image_in('pike-jp2.pdf')
     pim = PdfImage(xobj)
@@ -881,7 +890,7 @@ def test_invalid_icc(first_image_in):
         assert pim.icc is not None
 
 
-def test_dict_or_array_dict():
+def test_decodeparms_filter_alternates():
     pdf = pikepdf.new()
     imobj = Stream(
         pdf,
